@@ -4,7 +4,6 @@ import itertools
 import functools
 import os
 import uuid
-import warnings
 from abc import abstractmethod
 from contextlib import contextmanager
 
@@ -26,28 +25,20 @@ _AUTOLOGGING_TEST_MODE_ENV_VAR = "MLFLOW_AUTOLOGGING_TESTING"
 _AUTOLOGGING_PATCHES = {}
 
 
-def try_mlflow_log(fn, *args, **kwargs):
-    """
-    Catch exceptions and log a warning to avoid autolog throwing.
-    """
-    try:
-        return fn(*args, **kwargs)
-    except Exception as e:
-        if is_testing():
-            raise
-        else:
-            warnings.warn("Logging to MLflow failed: " + str(e), stacklevel=2)
-
-
 # Function attribute used for testing purposes to verify that a given function
-# has been wrapped with the `exception_safe_function` decorator
+# has been wrapped with the `exception_safe_function_for_class` and
+# `picklable_exception_safe_function` decorators
 _ATTRIBUTE_EXCEPTION_SAFE = "exception_safe"
 
 
-def exception_safe_function(function):
+def exception_safe_function_for_class(function):
     """
     Wraps the specified function with broad exception handling to guard
     against unexpected errors during autologging.
+    Note this function creates an unpicklable function as `safe_function` is locally defined,
+    but a class instance containing methods decorated by this function should be pickalable,
+    because pickle only saves instance attributes, not methods.
+    See https://docs.python.org/3/library/pickle.html#pickling-class-instances for more details.
     """
     if is_testing():
         setattr(function, _ATTRIBUTE_EXCEPTION_SAFE, True)
@@ -65,6 +56,27 @@ def exception_safe_function(function):
     return safe_function
 
 
+def _safe_function(function, *args, **kwargs):
+    try:
+        return function(*args, **kwargs)
+    except Exception as e:
+        if is_testing():
+            raise
+        else:
+            _logger.warning("Encountered unexpected error during autologging: %s", e)
+
+
+def picklable_exception_safe_function(function):
+    """
+    Wraps the specified function with broad exception handling to guard
+    against unexpected errors during autologging while preserving picklability.
+    """
+    if is_testing():
+        setattr(function, _ATTRIBUTE_EXCEPTION_SAFE, True)
+
+    return update_wrapper_extended(functools.partial(_safe_function, function), function)
+
+
 def _exception_safe_class_factory(base_class):
     """
     Creates an exception safe metaclass that inherits from `base_class`.
@@ -89,7 +101,7 @@ def _exception_safe_class_factory(base_class):
             for m in dct:
                 # class methods or static methods are not callable.
                 if callable(dct[m]):
-                    dct[m] = exception_safe_function(dct[m])
+                    dct[m] = exception_safe_function_for_class(dct[m])
             return base_class.__new__(cls, name, bases, dct)
 
     return _ExceptionSafeClass
@@ -213,20 +225,20 @@ def with_managed_run(autologging_integration, patch_function, tags=None):
 
             def _patch_implementation(self, original, *args, **kwargs):
                 if not mlflow.active_run():
-                    self.managed_run = try_mlflow_log(create_managed_run)
+                    self.managed_run = create_managed_run()
 
                 result = super(PatchWithManagedRun, self)._patch_implementation(
                     original, *args, **kwargs
                 )
 
                 if self.managed_run:
-                    try_mlflow_log(mlflow.end_run, RunStatus.to_string(RunStatus.FINISHED))
+                    mlflow.end_run(RunStatus.to_string(RunStatus.FINISHED))
 
                 return result
 
             def _on_exception(self, e):
                 if self.managed_run:
-                    try_mlflow_log(mlflow.end_run, RunStatus.to_string(RunStatus.FAILED))
+                    mlflow.end_run(RunStatus.to_string(RunStatus.FAILED))
                 super(PatchWithManagedRun, self)._on_exception(e)
 
         return PatchWithManagedRun
@@ -236,7 +248,7 @@ def with_managed_run(autologging_integration, patch_function, tags=None):
         def patch_with_managed_run(original, *args, **kwargs):
             managed_run = None
             if not mlflow.active_run():
-                managed_run = try_mlflow_log(create_managed_run)
+                managed_run = create_managed_run()
 
             try:
                 result = patch_function(original, *args, **kwargs)
@@ -245,11 +257,11 @@ def with_managed_run(autologging_integration, patch_function, tags=None):
                 # that runs are terminated if a user prematurely interrupts training execution
                 # (e.g. via sigint / ctrl-c)
                 if managed_run:
-                    try_mlflow_log(mlflow.end_run, RunStatus.to_string(RunStatus.FAILED))
+                    mlflow.end_run(RunStatus.to_string(RunStatus.FAILED))
                 raise
             else:
                 if managed_run:
-                    try_mlflow_log(mlflow.end_run, RunStatus.to_string(RunStatus.FINISHED))
+                    mlflow.end_run(RunStatus.to_string(RunStatus.FINISHED))
                 return result
 
         return patch_with_managed_run
@@ -310,6 +322,41 @@ def safe_patch(
     else:
         assert callable(patch_function)
 
+    original_fn = gorilla.get_original_attribute(
+        destination, function_name, bypass_descriptor_protocol=False
+    )
+    # Retrieve raw attribute while bypassing the descriptor protocol
+    raw_original_obj = gorilla.get_original_attribute(
+        destination, function_name, bypass_descriptor_protocol=True
+    )
+    if original_fn != raw_original_obj:
+        raise RuntimeError(f"Unsupport patch on {str(destination)}.{function_name}")
+    elif isinstance(original_fn, property):
+        is_property_method = True
+
+        # For property decorated methods (a kind of method delegation), e.g.
+        # class A:
+        #   @property
+        #   def f1(self):
+        #     ...
+        #     return delegated_f1
+        #
+        # suppose `a1` is an instance of class `A`,
+        # `A.f1.fget` will get the original `def f1(self)` method,
+        # and `A.f1.fget(a1)` will be equivalent to `a1.f1()` and
+        # its return value will be the `delegated_f1` function.
+        # So using the `property.fget` we can construct the (delegated) "original_fn"
+        def original(self, *args, **kwargs):
+            # the `original_fn.fget` will get the original method decorated by `property`
+            # the `original_fn.fget(self)` will get the delegated function returned by the
+            # property decorated method.
+            bound_delegate_method = original_fn.fget(self)
+            return bound_delegate_method(*args, **kwargs)
+
+    else:
+        original = original_fn
+        is_property_method = False
+
     def safe_patch_function(*args, **kwargs):
         """
         A safe wrapper around the specified `patch_function` implementation designed to
@@ -359,8 +406,6 @@ def safe_patch(
             if is_testing():
                 preexisting_run_for_testing = mlflow.active_run()
 
-            original = gorilla.get_original_attribute(destination, function_name)
-
             # Whether or not to exclude autologged content from user-created fluent runs
             # (i.e. runs created manually via `mlflow.start_run()`)
             exclusive = get_autologging_config(autologging_integration, "exclusive", False)
@@ -384,7 +429,8 @@ def safe_patch(
                 # warning behavior during original function execution, since autologging is being
                 # skipped
                 with set_non_mlflow_warnings_behavior_for_current_thread(
-                    disable_warnings=False, reroute_warnings=False,
+                    disable_warnings=False,
+                    reroute_warnings=False,
                 ):
                     return original(*args, **kwargs)
 
@@ -399,29 +445,60 @@ def safe_patch(
             failed_during_original = False
             # The active MLflow run (if any) associated with patch code execution
             patch_function_run_for_testing = None
+            # The exception raised during executing patching function
+            patch_function_exception = None
 
             def try_log_autologging_event(log_fn, *args):
                 try:
                     log_fn(*args)
                 except Exception as e:
                     _logger.debug(
-                        "Failed to log autologging event via '%s'. Exception: %s", log_fn, e,
+                        "Failed to log autologging event via '%s'. Exception: %s",
+                        log_fn,
+                        e,
                     )
 
+            def call_original_fn_with_event_logging(original_fn, og_args, og_kwargs):
+                try:
+                    try_log_autologging_event(
+                        AutologgingEventLogger.get_logger().log_original_function_start,
+                        session,
+                        destination,
+                        function_name,
+                        og_args,
+                        og_kwargs,
+                    )
+                    original_fn_result = original_fn(*og_args, **og_kwargs)
+
+                    try_log_autologging_event(
+                        AutologgingEventLogger.get_logger().log_original_function_success,
+                        session,
+                        destination,
+                        function_name,
+                        og_args,
+                        og_kwargs,
+                    )
+                    return original_fn_result
+                except Exception as original_fn_e:
+                    try_log_autologging_event(
+                        AutologgingEventLogger.get_logger().log_original_function_error,
+                        session,
+                        destination,
+                        function_name,
+                        og_args,
+                        og_kwargs,
+                        original_fn_e,
+                    )
+
+                    nonlocal failed_during_original
+                    failed_during_original = True
+                    raise
+
             with _AutologgingSessionManager.start_session(autologging_integration) as session:
                 try:
 
                     def call_original(*og_args, **og_kwargs):
-                        try:
-                            try_log_autologging_event(
-                                AutologgingEventLogger.get_logger().log_original_function_start,
-                                session,
-                                destination,
-                                function_name,
-                                og_args,
-                                og_kwargs,
-                            )
-
+                        def _original_fn(*_og_args, **_og_kwargs):
                             if is_testing():
                                 _validate_args(args, kwargs, og_args, og_kwargs)
                                 # By the time `original` is called by the patch implementation, we
@@ -443,34 +520,13 @@ def safe_patch(
                             # (`silent=True`), since these warnings originate from the ML framework
                             # or one of its dependencies and are likely relevant to the caller
                             with set_non_mlflow_warnings_behavior_for_current_thread(
-                                disable_warnings=False, reroute_warnings=False,
+                                disable_warnings=False,
+                                reroute_warnings=False,
                             ):
-                                original_result = original(*og_args, **og_kwargs)
+                                original_result = original(*_og_args, **_og_kwargs)
+                                return original_result
 
-                            try_log_autologging_event(
-                                AutologgingEventLogger.get_logger().log_original_function_success,
-                                session,
-                                destination,
-                                function_name,
-                                og_args,
-                                og_kwargs,
-                            )
-
-                            return original_result
-                        except Exception as e:
-                            try_log_autologging_event(
-                                AutologgingEventLogger.get_logger().log_original_function_error,
-                                session,
-                                destination,
-                                function_name,
-                                og_args,
-                                og_kwargs,
-                                e,
-                            )
-
-                            nonlocal failed_during_original
-                            failed_during_original = True
-                            raise
+                        return call_original_fn_with_event_logging(_original_fn, og_args, og_kwargs)
 
                     # Apply the name, docstring, and signature of `original` to `call_original`.
                     # This is important because several autologging patch implementations inspect
@@ -503,29 +559,13 @@ def safe_patch(
                     )
                 except Exception as e:
                     session.state = "failed"
-
+                    patch_function_exception = e
                     # Exceptions thrown during execution of the original function should be
                     # propagated to the caller. Additionally, exceptions encountered during test
                     # mode should be reraised to detect bugs in autologging implementations
                     if failed_during_original or is_testing():
                         raise
 
-                    try_log_autologging_event(
-                        AutologgingEventLogger.get_logger().log_patch_function_error,
-                        session,
-                        destination,
-                        function_name,
-                        args,
-                        kwargs,
-                        e,
-                    )
-
-                    _logger.warning(
-                        "Encountered unexpected error during %s autologging: %s",
-                        autologging_integration,
-                        e,
-                    )
-
                 if is_testing() and not preexisting_run_for_testing:
                     # If an MLflow run was created during the execution of patch code, verify that
                     # it is no longer active and that it contains expected autologging tags
@@ -536,13 +576,72 @@ def safe_patch(
                         _validate_autologging_run(
                             autologging_integration, patch_function_run_for_testing.info.run_id
                         )
+                try:
+                    if original_has_been_called:
+                        return original_result
+                    else:
+                        return call_original_fn_with_event_logging(original, args, kwargs)
+                finally:
+                    # If original function succeeds, but `patch_function_exception` exists,
+                    # it represent patching code unexpected failure, so we call
+                    # `log_patch_function_error` in this case.
+                    # If original function failed, we don't call `log_patch_function_error`
+                    # even if `patch_function_exception` exists, because original function failure
+                    # means there's some error in user code (e.g. user provide wrong arguments)
+                    if patch_function_exception is not None and not failed_during_original:
+                        try_log_autologging_event(
+                            AutologgingEventLogger.get_logger().log_patch_function_error,
+                            session,
+                            destination,
+                            function_name,
+                            args,
+                            kwargs,
+                            patch_function_exception,
+                        )
 
-                if original_has_been_called:
-                    return original_result
-                else:
-                    return original(*args, **kwargs)
+                        _logger.warning(
+                            "Encountered unexpected error during %s autologging: %s",
+                            autologging_integration,
+                            patch_function_exception,
+                        )
 
-    new_patch = _wrap_patch(destination, function_name, safe_patch_function)
+    if is_property_method:
+        # Create a patched function (also property decorated)
+        # like:
+        #
+        # class A:
+        # @property
+        # def get_bound_safe_patch_fn(self):
+        #   original_fn.fget(self) # do availability check
+        #   return bound_safe_patch_fn
+        #
+        # Suppose `a1` is instance of class A,
+        # then `a1.get_bound_safe_patch_fn(*args, **kwargs)` will be equivalent to
+        # `bound_safe_patch_fn(*args, **kwargs)`
+        def get_bound_safe_patch_fn(self):
+            # This `original_fn.fget` call is for availability check, if it raise error
+            # then `hasattr(obj, {func_name})` will return False
+            # so it mimic the original property behavior.
+            original_fn.fget(self)
+
+            def bound_safe_patch_fn(*args, **kwargs):
+                return safe_patch_function(self, *args, **kwargs)
+
+            # Make bound method `instance.target_method` keep the same doc and signature
+            bound_safe_patch_fn = update_wrapper_extended(bound_safe_patch_fn, original_fn.fget)
+            # Here return the bound safe patch function because user call property decorated
+            # method will like `instance.property_decorated_method(...)`, and internally it will
+            # call the `bound_safe_patch_fn`, the argument list don't include the `self` argument,
+            # so return bound function here.
+            return bound_safe_patch_fn
+
+        # Make unbound method `class.target_method` keep the same doc and signature
+        get_bound_safe_patch_fn = update_wrapper_extended(get_bound_safe_patch_fn, original_fn.fget)
+        safe_patch_obj = property(get_bound_safe_patch_fn)
+    else:
+        safe_patch_obj = update_wrapper_extended(safe_patch_function, original)
+
+    new_patch = _wrap_patch(destination, function_name, safe_patch_obj)
     _store_patch(autologging_integration, new_patch)
 
 
@@ -621,22 +720,20 @@ def update_wrapper_extended(wrapper, wrapped):
     return updated_wrapper
 
 
-def _wrap_patch(destination, name, patch, settings=None):
+def _wrap_patch(destination, name, patch_obj, settings=None):
     """
-    Apply a patch while preserving the attributes (e.g. __doc__) of an original function.
+    Apply a patch.
 
     :param destination: Patch destination
     :param name: Name of the attribute at the destination
-    :param patch: Patch function
+    :param patch_obj: Patch object, it should be a function or a property decorated function
+                      to be assigned to the patch point {destination}.{name}
     :param settings: Settings for gorilla.Patch
     """
     if settings is None:
         settings = gorilla.Settings(allow_hit=True, store_hit=True)
 
-    original = getattr(destination, name)
-    wrapped = update_wrapper_extended(patch, original)
-
-    patch = gorilla.Patch(destination, name, wrapped, settings=settings)
+    patch = gorilla.Patch(destination, name, patch_obj, settings=settings)
     gorilla.apply(patch)
     return patch
 
@@ -686,8 +783,9 @@ def _validate_args(
         - All arguments supplied to the patched ML function are forwarded to the
           original ML function
         - Any additional arguments supplied to the original function are exception safe (i.e.
-          they are either functions decorated with the `@exception_safe_function` decorator
-          or classes / instances of classes with type `ExceptionSafeClass`
+          they are either functions decorated with the `@exception_safe_function_for_class` or
+          `@pickalable_exception_safe_function` decorators, or classes / instances of classes with
+          type `ExceptionSafeClass`
     """
 
     def _validate_new_input(inp):
@@ -695,7 +793,8 @@ def _validate_args(
         Validates a new input (arg or kwarg) introduced to the underlying / original ML function
         call during the execution of a patched ML function. The new input is valid if:
 
-            - The new input is a function that has been decorated with `exception_safe_function`
+            - The new input is a function that has been decorated with
+              `exception_safe_function_for_class` or `pickalable_exception_safe_function`
             - OR the new input is a class with the `ExceptionSafeClass` metaclass
             - OR the new input is a list and each of its elements is valid according to the
               these criteria
@@ -706,8 +805,9 @@ def _validate_args(
         elif callable(inp):
             assert getattr(inp, _ATTRIBUTE_EXCEPTION_SAFE, False), (
                 "New function argument '{}' passed to original function is not exception-safe."
-                " Please decorate the function with `exception_safe_function`.".format(inp)
-            )
+                " Please decorate the function with `exception_safe_function` or "
+                "`pickalable_exception_safe_function`"
+            ).format(inp)
         else:
             assert hasattr(inp, "__class__") and type(inp.__class__) in [
                 ExceptionSafeClass,
@@ -715,9 +815,9 @@ def _validate_args(
             ], (
                 "Invalid new input '{}'. New args / kwargs introduced to `original` function "
                 "calls by patched code must either be functions decorated with "
-                "`exception_safe_function`, instances of classes with the `ExceptionSafeClass` "
-                "or `ExceptionSafeAbstractClass` metaclass safe or lists of such exception safe "
-                "functions / classes.".format(inp)
+                "`exception_safe_function_for_class`, instances of classes with the "
+                "`ExceptionSafeClass` or `ExceptionSafeAbstractClass` metaclass safe or lists of "
+                "such exception safe functions / classes.".format(inp)
             )
 
     def _validate(autologging_call_input, user_call_input=None):
@@ -746,9 +846,10 @@ def _validate_args(
 
         if type(autologging_call_input) in [list, tuple]:
             length_difference = len(autologging_call_input) - len(user_call_input)
-            assert length_difference >= 0, (
-                "{} expected inputs are missing from the call"
-                " to the original function.".format(length_difference)
+            assert (
+                length_difference >= 0
+            ), "{} expected inputs are missing from the call" " to the original function.".format(
+                length_difference
             )
             # If the autologging call input is longer than the user call input, we `zip_longest`
             # will pad the user call input with `None` values to ensure that the subsequent calls
@@ -778,10 +879,10 @@ def _validate_args(
 
 
 __all__ = [
-    "try_mlflow_log",
     "safe_patch",
     "is_testing",
-    "exception_safe_function",
+    "exception_safe_function_for_class",
+    "picklable_exception_safe_function",
     "ExceptionSafeClass",
     "ExceptionSafeAbstractClass",
     "PatchFunction",
